; 7-bit ASCII Keyboard translation program.  File: keyboard.asm
;
; Global Assembler directives
	list		c=132, n=80
	list		p=16F883
	errorlevel	-302	; Disable messages for register banking
	errorlevel	-306	; Disable messages for page crossing
;
#include	p16f883.inc
;
; This program interfaces an ancient parallel keyboard to a system via a serial interface.
; It translates key presses into serial characters, handles a repeat function, driving an
; external indicator for shift and generating a bell sound when the BEL character is received.
; Bell sound is generated using 7-bit unsigned PCM values, sampled at 3906.25 Hz driving a
; 62500Hz PWM Output to be externally filtered, amplified and output by a speaker.
;
; Copyright (c) 2016 Dan Julio.  All rights reserved.
;
; Released under the GNU Affero General Public License
; Version 3, 19 November 2007
; See http://choosealicense.com/licenses/agpl-3.0/
;
;
; Revision History
;   07/13/16 Revision 1.0  DJJ  Initial
;
; ------------------------------------------------------------------------
; Processor Information:
;
; Target processor: PIC 16F883
; Processor frequency: Internal 8 MHz (500 ns instruction cycle)
;
; Program Memory Layout
;    Page 0: 0x0000 - 0x077F    Main code, Interrupt handler and subroutines
;    Page 0: 0x0780 - 0x07FF    ASCII lookup table
;    Page 1: 0x0800 - 0x0FFF    Bell Sound table
;
; Input/Output Pin Assignments (DIP package)
;
;   Pin     Signal                       Polarity      I/O     Default State
;   ------------------------------------------------------------------------
;    1 RE1  MCLR/VPP                     Active Low    Input   0
;    2 RA0  D0                           Active High   Input   0
;    3 RA1  D1                           Active High   Input   0
;    4 RA2  D2                           Active High   Input   0
;    5 RA3  D3                           Active High   Input   0
;    6 RA4  D4                           Active High   Input   0
;    7 RA5  D5                           Active High   Input   0
;   10 RA6  D6                           Active High   Input   0
;    9 RA7  Odd Parity                   Active High   Input   0
;   21 RB0  Strobe (INT)                 Active High   Input   0 
;   22 RB1  Shift Input                  Active High   Input   0
;   23 RB2  Reserved                     -             Input   0 weak pull-up
;   24 RB3  Reserved                     -             Input   0 weak pull-up
;   25 RB4  Reserved                     -             Input   0 weak pull-up
;   26 RB5  Reserved                     -             Input   0 weak pull-up
;   27 RB6  ICSPCLK                      -             Input   0 weak pull-up
;   28 RB7  ICSPDAT                      -             Input   0 weak pull-up
;   11 RC0  Shift Output                 Active High   Output  0
;   12 RC1  Reserved                     Active High   Output  0
;   13 RC2  PWM Output                   Active High   Output  0
;   14 RC3  Reserved                     Active High   Output  0
;   15 RC4  Reserved                     Active High   Output  0
;   16 RC5  Reserved                     Active High   Output  0
;   17 RC6  Serial TX                    Active Low    Output  0
;   18 RC7  Serial RX                    Active Low    Input   0
;
; TIMER Utilization
; -----------------
; TIMER0 : Generates tone sample clock
; TIMER1 : Generates key repeat clock
; TIMER2 : Supports CCP1 PWM output for bell tone generation
;
; ------------------------------------------------------------------------
; Set the device configuration bits
;  CONFIG1
;    DEBUG = 1 (RB6/7 are general purpose IO pins)
;    LVP = 0 (RB3 is digital IO)
;    FCMEN  = 0 (Fail-Safe Clock Monitor disabled)
;    IESO = 0 (Internal/External Switchover mode is disabled)
;    BOR = 11 (Brown-out detection enabled)
;    CPD = 1 (Data memory code protection disabled)
;    CP = 1 (Program memory code protection disabled)
;    MCLRE = 1 (Re3/MCLK pin function is MCLR)
;    PWRTE = 1 (Power-up Timer enabled)
;    WDTE = 1 (WDT enabled)
;    FOSC2:0 = 100 (INTOSCIO)
;  CONFIG2
;    WRT1:0 = 11 (Flash Program memory Write protection off)
;    BOR4V = 1 (Brown-out voltage = 4.0V)

        __config _CONFIG1, _DEBUG_OFF & _LVP_OFF & _FCMEN_OFF & _IESO_OFF & _BOR_ON & _CPD_OFF & _CP_OFF & _MCLRE_ON & _PWRTE_ON & _WDT_ON & _INTOSCIO

        __config _CONFIG2, _WRT_OFF & _BOR40V


; ------------------------------------------------------------------------
; Program Constants
;

;
; External device bits (port number bits)
KB_STROBE	equ	.0
SHIFT_IN	equ	.1
SHIFT_OUT	equ	.0


;
; FIFO Masks (power-of-two for 16-entry fifos)
#define TX_FIFO_MASK	0x0F
#define RX_FIFO_MASK	0x0F

;
; Special characters
#define BELL_CHAR	0x07
#define REPEAT_CHAR	0x05

;
; Flags bits (active high)
ShiftDown	equ	.0
RepeatActive	equ	.1
BellInProgress	equ	.2
UseOddSample	equ	.3


;
; Variable register equates
TempReg		equ	0x20
TxPushPtr	equ	0x21
TxPopPtr	equ	0x22
RxPushPtr	equ	0x23
RxPopPtr	equ	0x24
Flags		equ	0x25
RepeatChar	equ	0x26
SoundIndexHi	equ	0x27
SoundIndexLo	equ	0x28
SoundOddSample	equ	0x29


; 16-byte Serial TX fifo
TxFifoStart	equ	0x30

; 16-byte Serial RX fifo
RxFifoStart	equ	0x40

;
; Registers shared among all banks for easy access
;
; Flash access registers
FlashAddrHi	equ	0x70
FlashDataHi	equ	0x71
FlashDataLo	equ	0x72

;
; Interrupt state save registers
FSR_Temp        equ     0x7C
STATUS_Temp     equ     0x7D
PCLATH_Temp     equ     0x7E
W_Temp          equ     0x7F


;
; Initialization constants
STATUS_SETUP	equ	B'00011000'	; 7: IRP = 0 (page 0-1)
					; 6:5: RP1:0 = 00b (page 0)
					; 4: TO = 1 (default state)
					; 3: PD = 1 (default state)
					; 2: Z = 0
					; 1: DC = 0
					; 0: C = 0

OSCCON_SETUP	equ	B'01110000'	; 7: Unimplemented
					; 6:4: IRCF = 111 (8 MHz internal select)
					; 3: OSTS
					; 2: HTS
					; 1: LTS
					; 0: SCS = 0 (Use CONFIG1 FOSC[2:0])

OPTION_SETUP	equ	B'00000000'	; 7: RBPU = 0 (enable pull-ups)
					; 6: INTEDG = 0 (Int on falling edge RB0/INT)
					; 5: T0CS = 0 (internal TMR0 clock)
					; 4: T0SE = 0 (rising TMR0 clock)
					; 3: PSA = 0 (prescaler to TMR0)
					; 2:0: prescaler = 000b (1:2)
					; TIMER2 clock = 1 MHz ==> roll over @ 3906.25 Hz
					;   for sound sample rate

INTCON_SETUP	equ	B'11110000'	; 7: GIE = 1 (Interrupts enabled)
					; 6: PEIE = 1 (Peripheral ints enabled)
					; 5: T0IE = 1 (TMR0 ints enabled)
					; 4: INTE = 1 (INT ints enabled)
					; 3: RBIE = 0 (RB ints disabled)
					; 2: T0IF = 0
					; 1: INTF = 0
					; 0: RBIF = 0

PIE1_INIT	equ	B'00100000'	; 7: Unimplemented
					; 6: ADIE = 0
					; 5: RCIE = 1 (Serial RX interrupt enabled)
					; 4: TXIE = 0 (Serial TX interrupt disabled)
					; 3: SSPIE = 0
					; 2: CCP1IE = 0
					; 1: TMR2IE = 0
					; 0: TMR1IE = 0

PORTA_INIT	equ	B'00000000'	; No outputs enabled
PORTB_INIT	equ	B'00000000'	; No outputs enabled
PORTC_INIT	equ	B'00000000'	; No outputs enabled
PORTA_OE	equ	B'11111111'	; All inputs
PORTB_OE	equ	B'11111111'	; All inputs
PORTC_OE	equ	B'10000000'	; Only RC[7] input

ANSEL_INIT	equ	B'00000000'	; All PORTA bits are digital I/O

ANSELH_INIT	equ	B'00000000'	; All PORTB bits are digital I/O

WPUB_INIT	equ	B'11111100'	; Pull-up's enabled on unused bits

IOCB_INIT	equ	B'00000000'	; Interrupt-on-change disabled

TXSTA_INIT	equ	B'00100100'	; 7: CSRC = 0 (Don't care)
					; 6: TX9 = 0 (8-bit transmission)`
					; 5: TXEN = 1 (transmit enabled)
					; 4: SYNC = 0 (asynch mode)
					; 3: SENDB = 0 (Send Sync Break = 0)
					; 2: BRGH = 1 (high speed mode)
					; 1: TRMT = 0
					; 0: TX9D = 0 (9th xmit data)

RCSTA_INIT	equ	B'10010000'	; 7: SPEN = 1 (Serial port enabled)
					; 6: RX9 = 0 (8-bit reception)
					; 5: SREN = 0 (Don't care)
					; 4: CREN = 1 (enable continuous)
					; 3: ADDEN = 0 (disable addr detection)
					; 2: FERR = 0
					; 1: OERR = 0
					; 0: RX9D = 0

BAUDCTL_INIT	equ	B'00001000'	; 7: ABDOVF
					; 6: RCIDL
					; 5: Unimplemented
					; 4: SCKP = 0 (non-inverted TX data)
					; 3: BRG16 = 1
					; 2: Unimplemented
					; 1: WUE = 0
					; 0: ABDEN = 0 (auto-baud disabled)

; 57600 baud
SPBRG_INIT		equ	.34

; TIMER1 clock set to 250 kHz (16-bit rollover at about 262 mSec/3.8 repeats/sec)
T1CON_INIT	equ	B'00110001'	; 7: T1GINV = 0
					; 6: TMR1GE = 0 (TIMER1 always counting)
					; 5:4: T1CKPS<1:0> = 11 (1:8 Prescale)
					; 3: T1OSCEN = 0 (LP Osc off)
					; 2: T1SYNCN = 0
					; 1: TMR1CS = 0 (Int clock Fosc/4)
					; 0: TMR1ON = 1

; TIMER2 clock set to 2 MHz and PR2 to 0x1F for PWM frequency of 62500 Hz
; and 7-bit PWM value
T2CON_INIT	equ	B'00000100'	; 7: Unimplemented
					; 6:3: TOUTPS<3:0> = 0000
					; 2: TMR2ON = 1
					; 1:0: T2CKPS<1:0> = 00 (1:1 Prescale)

PR2_INIT	equ	0x1F

CCP1CON_INIT	equ	B'00001100'	; 7:6: P1M<1:0> = 00 (Single output P1A output)
					; 5:4: DC1B<1:0> = 00
					; 3:0: CCP1M<3:0> = 1100 (P1A active high)

CCP1CON_MASK	equ	0xCF		; Masks control bits (leaving PWM LSbits)



; ------------------------------------------------------------------------
; Program EEPROM Initialization
;

; ASCII_TABLE translates the 7-bit codes from the keyboard to reasonable ASCII values
;
ASCII_TABLE	org	0x0780
	data	'2', '7', '1', '9', '6', 0x05, 0x06, '3'
	data	0x08, 0x09, 0x0a, 0x0b, 0x2C, 0x0d, 0x0e, 0x0f
	data	'8', 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, '5'
	data	'4', '0', '.', 0x04, 0x07, 0x0e, 0x0f, 0x0c
	data	0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27
	data	0x28, 0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e, 0x2f
	data	0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37
	data	0x38, 0x39, 0x3a, 0x3b, 0x3c, 0x3d, 0x3e, 0x3f
	data	0x40, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47
	data	0x48, 0x49, 0x4a, 0x4b, 0x4c, 0x4d, 0x4e, 0x4f
	data	0x50, 0x51, 0x52, 0x53, 0x54, 0x55, 0x56, 0x57
	data	0x58, 0x59, 0x5a, 0x5b, 0x5c, 0x5d, 0x5e, 0x5f
	data	0x60, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67
	data	0x68, 0x69, 0x6a, 0x6b, 0x6c, 0x6d, 0x6e, 0x6f
	data	0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77
	data	0x78, 0x79, 0x7a, 0x7b, 0x7c, 0x7d, 0x7e, 0x7f


;
; SOUND_TABLE contains the 7-bit unsigned PCM data for the bell sound and may be up to
; 2K entries long for 4K samples with are played back at 3906.25 Hz.  Samples are
; packed two per 14-bit instruction word with the even address in bits 6:0 and the odd
; address in bits 13:7.  The last sample should be 0 to disable the audio drive
; transistor.  In fact be sure to ramp the PWM down to 0 after the sound is done
; to avoid an audible thump.
;
SOUND_TABLE_LEN	equ	.1848
SOUND_TABLE 	org	0x0800
	data	0x1fbf, 0x203f, 0x2040, 0x2040, 0x36da, 0x1f5f, 0x0a1d, 0x25a7
	data	0x3567, 0x1650, 0x0d15, 0x2db7, 0x306c, 0x0f3f, 0x1313, 0x3349
	data	0x28ea, 0x0b2d, 0x1b9a, 0x35d9, 0x2060, 0x099f, 0x24a6, 0x3566
	data	0x1752, 0x0c15, 0x2d36, 0x306b, 0x0fbf, 0x1213, 0x32c7, 0x29ea
	data	0x0b2f, 0x1a99, 0x35d8, 0x20e1, 0x0a20, 0x23a4, 0x3565, 0x1853
	data	0x0c96, 0x2bb4, 0x316b, 0x10c2, 0x1214, 0x3246, 0x2a6a, 0x0bb0
	data	0x1998, 0x3557, 0x21e2, 0x0a21, 0x22a3, 0x3563, 0x18d4, 0x0c17
	data	0x2b33, 0x316a, 0x1144, 0x1194, 0x31c4, 0x2aea, 0x0c32, 0x1918
	data	0x3555, 0x2263, 0x0a23, 0x21a2, 0x3562, 0x19d6, 0x0c18, 0x2a31
	data	0x31e9, 0x1245, 0x1115, 0x30c3, 0x2b6a, 0x0cb4, 0x1898, 0x34d3
	data	0x2363, 0x0aa4, 0x2121, 0x3561, 0x1ad7, 0x0c1a, 0x29b0, 0x3269
	data	0x12c7, 0x1095, 0x3041, 0x2bea, 0x0d35, 0x1797, 0x3452, 0x2464
	data	0x0aa6, 0x2020, 0x355f, 0x1b58, 0x0b9b, 0x28ae, 0x3268, 0x13c8
	data	0x1015, 0x2fbf, 0x2cea, 0x0db7, 0x1717, 0x33d0, 0x24e4, 0x0b27
	data	0x1f20, 0x34de, 0x1c59, 0x0b9c, 0x27ad, 0x32e7, 0x144a, 0x0f96
	data	0x2ebe, 0x2d69, 0x0e39, 0x1697, 0x334e, 0x25e5, 0x0b29, 0x1e9f
	data	0x34dd, 0x1d5a, 0x0b9d, 0x272c, 0x32e6, 0x154b, 0x0f97, 0x2e3c
	data	0x2de9, 0x0f3a, 0x1597, 0x32cd, 0x2665, 0x0bab, 0x1d9e, 0x34db
	data	0x1ddb, 0x0b9e, 0x262b, 0x32e5, 0x15cd, 0x0f17, 0x2d3b, 0x2e69
	data	0x0fbc, 0x1517, 0x324b, 0x26e5, 0x0c2c, 0x1d1e, 0x345a, 0x1edc
	data	0x0b9f, 0x252a, 0x32e4, 0x16ce, 0x0e98, 0x2cb9, 0x2e68, 0x103e
	data	0x1497, 0x31ca, 0x27e6, 0x0cae, 0x1c1d, 0x3458, 0x1f5d, 0x0ba1
	data	0x24a8, 0x3363, 0x17cf, 0x0e99, 0x2c38, 0x2ee8, 0x10bf, 0x1418
	data	0x3148, 0x2866, 0x0caf, 0x1b9d, 0x33d7, 0x205e, 0x0c22, 0x23a7
	data	0x3362, 0x1851, 0x0e1a, 0x2b36, 0x2f67, 0x11c1, 0x1398, 0x30c6
	data	0x28e6, 0x0d31, 0x1a9c, 0x33d5, 0x215e, 0x0c23, 0x2326, 0x3361
	data	0x1952, 0x0e1b, 0x2a35, 0x2fe6, 0x1242, 0x1318, 0x3045, 0x2966
	data	0x0db2, 0x1a1c, 0x3354, 0x21df, 0x0c25, 0x2226, 0x335f, 0x19d3
	data	0x0e1c, 0x29b3, 0x2fe6, 0x1344, 0x1298, 0x2fc3, 0x29e6, 0x0e34
	data	0x199c, 0x32d2, 0x225f, 0x0ca6, 0x2125, 0x335e, 0x1ad4, 0x0e1d
	data	0x28b2, 0x3065, 0x13c5, 0x1219, 0x2f42, 0x2a66, 0x0eb6, 0x191c
	data	0x32d1, 0x2360, 0x0ca8, 0x20a4, 0x32dd, 0x1b55, 0x0d9e, 0x2831
	data	0x3064, 0x1447, 0x1219, 0x2e40, 0x2ae5, 0x0f37, 0x181b, 0x324f
	data	0x23e0, 0x0d29, 0x1fa3, 0x32dc, 0x1c56, 0x0d9f, 0x2730, 0x30e3
	data	0x1548, 0x119a, 0x2dbf, 0x2b65, 0x0fb9, 0x179b, 0x31ce, 0x24e1
	data	0x0d2b, 0x1f23, 0x32da, 0x1cd7, 0x0da0, 0x26af, 0x30e2, 0x15c9
	data	0x111b, 0x2d3d, 0x2be5, 0x10ba, 0x171b, 0x314c, 0x2561, 0x0dac
	data	0x1ea2, 0x32d9, 0x1dd8, 0x0da1, 0x25ae, 0x30e2, 0x16ca, 0x111b
	data	0x2c3c, 0x2c64, 0x113c, 0x169b, 0x30cb, 0x25e1, 0x0e2d, 0x1da2
	data	0x3258, 0x1e58, 0x0e22, 0x252d, 0x30e1, 0x174c, 0x109c, 0x2bbb
	data	0x2ce4, 0x11bd, 0x161c, 0x3049, 0x2661, 0x0e2f, 0x1d21, 0x3256
	data	0x1f59, 0x0e23, 0x24ac, 0x30e0, 0x184d, 0x109d, 0x2b39, 0x2d63
	data	0x123e, 0x159c, 0x2fc8, 0x26e1, 0x0eb0, 0x1ca1, 0x31d5, 0x1fda
	data	0x0e25, 0x23ab, 0x30de, 0x18ce, 0x109e, 0x2a38, 0x2d63, 0x12c0
	data	0x151c, 0x2f46, 0x27e1, 0x0f32, 0x1ba0, 0x31d4, 0x20da, 0x0e26
	data	0x232a, 0x30dd, 0x194f, 0x101e, 0x29b7, 0x2de2, 0x13c1, 0x149c
	data	0x2ec5, 0x2861, 0x0fb3, 0x1b20, 0x3152, 0x215b, 0x0ea7, 0x2229
	data	0x30dc, 0x1a50, 0x101f, 0x2936, 0x2de2, 0x1443, 0x149d, 0x2e44
	data	0x28e1, 0x1035, 0x1aa0, 0x30d1, 0x21db, 0x0ea9, 0x21a8, 0x30db
	data	0x1ad1, 0x1020, 0x2834, 0x2e61, 0x14c4, 0x141d, 0x2dc2, 0x28e1
	data	0x10b6, 0x1a20, 0x30cf, 0x225c, 0x0f2a, 0x2128, 0x30da, 0x1bd2
	data	0x1021, 0x27b3, 0x2e60, 0x1545, 0x139e, 0x2cc1, 0x2961, 0x1137
	data	0x19a0, 0x304e, 0x235c, 0x0f2b, 0x2027, 0x30d9, 0x1c53, 0x1022
	data	0x26b2, 0x2e5f, 0x1646, 0x139e, 0x2c40, 0x29e1, 0x11b9, 0x1920
	data	0x2fcd, 0x23dc, 0x0fac, 0x1fa7, 0x3058, 0x1cd3, 0x1023, 0x2631
	data	0x2edf, 0x16c7, 0x131f, 0x2bbe, 0x2a60, 0x123a, 0x18a0, 0x2f4b
	data	0x245d, 0x0fae, 0x1f26, 0x3056, 0x1dd4, 0x1024, 0x25b0, 0x2ede
	data	0x1748, 0x1320, 0x2b3d, 0x2ae0, 0x12bb, 0x1820, 0x2eca, 0x24dd
	data	0x102f, 0x1e26, 0x3055, 0x1e55, 0x1025, 0x24af, 0x2edd, 0x184a
	data	0x12a0, 0x2abc, 0x2adf, 0x133d, 0x17a0, 0x2e49, 0x255d, 0x10b0
	data	0x1da5, 0x2fd4, 0x1ed5, 0x1027, 0x242f, 0x2edc, 0x18cb, 0x12a1
	data	0x29bb, 0x2b5f, 0x13be, 0x1720, 0x2dc7, 0x25dd, 0x10b2, 0x1d25
	data	0x2fd3, 0x1f56, 0x10a8, 0x23ae, 0x2edb, 0x194b, 0x12a2, 0x293a
	data	0x2b5e, 0x143f, 0x1721, 0x2d46, 0x265d, 0x1133, 0x1ca5, 0x2f52
	data	0x2057, 0x10a9, 0x232d, 0x2eda, 0x19cc, 0x12a3, 0x28b8, 0x2bde
	data	0x14c0, 0x16a1, 0x2cc5, 0x26dd, 0x11b4, 0x1c25, 0x2f50, 0x20d7
	data	0x10aa, 0x222d, 0x2ed9, 0x1acd, 0x12a3, 0x2837, 0x2bdd, 0x15c1
	data	0x1621, 0x2c44, 0x275d, 0x1235, 0x1ba4, 0x2ecf, 0x2157, 0x112b
	data	0x21ac, 0x2ed8, 0x1b4e, 0x1224, 0x2736, 0x2bdd, 0x1643, 0x1622
	data	0x2bc2, 0x275c, 0x12b7, 0x1b24, 0x2e4e, 0x21d8, 0x112c, 0x212b
	data	0x2e57, 0x1bcf, 0x1225, 0x26b6, 0x2c5c, 0x16c4, 0x15a2, 0x2b41
	data	0x27dc, 0x1338, 0x1aa4, 0x2dcd, 0x2258, 0x11ad, 0x20ab, 0x2e56
	data	0x1c50, 0x1226, 0x2635, 0x2c5b, 0x1745, 0x15a3, 0x2ac0, 0x285c
	data	0x1339, 0x1a24, 0x2dcb, 0x22d8, 0x11af, 0x202a, 0x2e55, 0x1d50
	data	0x1227, 0x25b4, 0x2c5a, 0x17c6, 0x1523, 0x2a3f, 0x285c, 0x13ba
	data	0x19a5, 0x2d4a, 0x2358, 0x1230, 0x1faa, 0x2dd4, 0x1dd1, 0x12a8
	data	0x2533, 0x2c5a, 0x1847, 0x1524, 0x29be, 0x28db, 0x143b, 0x19a5
	data	0x2cc9, 0x23d8, 0x1231, 0x1f2a, 0x2dd3, 0x1e51, 0x12a9, 0x24b2
	data	0x2c59, 0x18c7, 0x1525, 0x293d, 0x295b, 0x14bc, 0x1925, 0x2c48
	data	0x2458, 0x12b2, 0x1ea9, 0x2dd1, 0x1ed2, 0x12aa, 0x23b2, 0x2c58
	data	0x19c8, 0x14a5, 0x28bc, 0x295a, 0x153e, 0x18a5, 0x2bc7, 0x24d8
	data	0x1333, 0x1e29, 0x2d50, 0x1f52, 0x12ab, 0x2331, 0x2c57, 0x1a49
	data	0x14a6, 0x283b, 0x295a, 0x15bf, 0x18a5, 0x2bc6, 0x2558, 0x1334
	data	0x1da9, 0x2d4f, 0x1fd3, 0x132c, 0x22b0, 0x2c56, 0x1aca, 0x14a7
	data	0x27ba, 0x29d9, 0x1640, 0x1826, 0x2b45, 0x2558, 0x13b5, 0x1d29
	data	0x2cce, 0x2053, 0x132d, 0x2230, 0x2c55, 0x1b4b, 0x14a8, 0x2739
	data	0x29d9, 0x16c1, 0x1826, 0x2ac4, 0x25d8, 0x1436, 0x1ca9, 0x2c4d
	data	0x20d3, 0x132e, 0x21af, 0x2c54, 0x1bcb, 0x14a8, 0x26b8, 0x29d8
	data	0x1741, 0x17a7, 0x2a43, 0x2658, 0x14b7, 0x1c29, 0x2c4c, 0x2153
	data	0x13af, 0x212f, 0x2c53, 0x1c4c, 0x14a9, 0x2638, 0x2a57, 0x17c2
	data	0x17a7, 0x29c2, 0x2657, 0x14b9, 0x1ba9, 0x2bcb, 0x21d4, 0x13b0
	data	0x20af, 0x2bd3, 0x1ccc, 0x14aa, 0x25b7, 0x2a57, 0x1843, 0x1728
	data	0x2941, 0x26d7, 0x153a, 0x1ba9, 0x2b4a, 0x2254, 0x1431, 0x202e
	data	0x2bd2, 0x1d4d, 0x14ab, 0x24b6, 0x2a56, 0x18c4, 0x1728, 0x28c0
	data	0x26d7, 0x15bb, 0x1b29, 0x2b49, 0x2254, 0x1432, 0x1fae, 0x2bd1
	data	0x1dcd, 0x14ac, 0x2436, 0x2a55, 0x1945, 0x1729, 0x283f, 0x2756
	data	0x163c, 0x1aa9, 0x2ac8, 0x22d4, 0x14b3, 0x1f2e, 0x2b50, 0x1e4e
	data	0x14ac, 0x23b5, 0x2a55, 0x19c6, 0x1729, 0x27be, 0x2756, 0x16bc
	data	0x1aaa, 0x2a47, 0x2354, 0x14b4, 0x1f2e, 0x2b4f, 0x1ece, 0x152d
	data	0x23b4, 0x2a54, 0x1a46, 0x172a, 0x273d, 0x2756, 0x173d, 0x1a2a
	data	0x2a46, 0x23d4, 0x1535, 0x1ead, 0x2ace, 0x1f4f, 0x152e, 0x2334
	data	0x2a53, 0x1ac7, 0x16aa, 0x26bc, 0x27d5, 0x17be, 0x1a2a, 0x29c5
	data	0x23d4, 0x15b6, 0x1e2d, 0x2acd, 0x1fcf, 0x152f, 0x22b3, 0x2a52
	data	0x1b48, 0x16ab, 0x263c, 0x27d5, 0x17bf, 0x19aa, 0x2944, 0x2454
	data	0x15b7, 0x1dad, 0x2a4c, 0x204f, 0x15b0, 0x2233, 0x29d2, 0x1bc8
	data	0x16ac, 0x25bb, 0x27d4, 0x1840, 0x19ab, 0x28c3, 0x2453, 0x1638
	data	0x1dad, 0x2a4b, 0x204f, 0x15b1, 0x21b3, 0x29d1, 0x1c49, 0x16ac
	data	0x25ba, 0x27d4, 0x18c1, 0x19ab, 0x2842, 0x24d3, 0x16b9, 0x1d2d
	data	0x29ca, 0x20cf, 0x15b2, 0x2132, 0x29d0, 0x1cc9, 0x16ad, 0x253a
	data	0x27d3, 0x1941, 0x192c, 0x2842, 0x24d3, 0x16b9, 0x1cad, 0x29c9
	data	0x2150, 0x1633, 0x20b2, 0x29cf, 0x1d4a, 0x16ae, 0x24b9, 0x2852
	data	0x19c2, 0x192c, 0x27c1, 0x2553, 0x173a, 0x1cae, 0x2948, 0x2150
	data	0x1633, 0x2032, 0x294e, 0x1dca, 0x172f, 0x2438, 0x2852, 0x1a43
	data	0x192d, 0x2740, 0x2552, 0x17bb, 0x1c2e, 0x28c7, 0x21d0, 0x16b4
	data	0x2032, 0x294e, 0x1e4a, 0x172f, 0x23b8, 0x2851, 0x1ac3, 0x192d
	data	0x26bf, 0x25d2, 0x183c, 0x1c2e, 0x28c7, 0x2250, 0x16b5, 0x1fb2
	data	0x294d, 0x1e4b, 0x1730, 0x2338, 0x2851, 0x1ac4, 0x192e, 0x263f
	data	0x25d1, 0x183d, 0x1bae, 0x2846, 0x2250, 0x1736, 0x1f31, 0x28cc
	data	0x1ecb, 0x1731, 0x22b7, 0x27d0, 0x1b45, 0x18ae, 0x263e, 0x25d1
	data	0x18bd, 0x1bae, 0x2845, 0x22cf, 0x1737, 0x1f31, 0x28cb, 0x1f4b
	data	0x17b2, 0x22b7, 0x27cf, 0x1bc5, 0x18af, 0x25bd, 0x25d1, 0x193e
	data	0x1baf, 0x27c4, 0x22cf, 0x17b8, 0x1eb1, 0x284a, 0x1f4b, 0x17b2
	data	0x2236, 0x27cf, 0x1c46, 0x18af, 0x253d, 0x25d0, 0x19bf, 0x1b2f
	data	0x2744, 0x234f, 0x1838, 0x1eb1, 0x284a, 0x1fcb, 0x17b3, 0x21b6
	data	0x27ce, 0x1cc6, 0x18b0, 0x24bc, 0x2650, 0x19bf, 0x1b2f, 0x2743
	data	0x234f, 0x1839, 0x1e31, 0x27c9, 0x204c, 0x1834, 0x2136, 0x27cd
	data	0x1cc6, 0x18b0, 0x243c, 0x264f, 0x1a40, 0x1b30, 0x26c2, 0x23cf
	data	0x18ba, 0x1db2, 0x27c8, 0x204c, 0x1834, 0x2136, 0x27cc, 0x1d47
	data	0x1931, 0x243b, 0x264f, 0x1ac1, 0x1b30, 0x2641, 0x23ce, 0x18ba
	data	0x1db2, 0x2747, 0x20cc, 0x1835, 0x20b5, 0x274c, 0x1dc7, 0x1932
	data	0x23bb, 0x264e, 0x1b41, 0x1ab1, 0x25c1, 0x23ce, 0x193b, 0x1db2
	data	0x2747, 0x20cc, 0x18b6, 0x20b5, 0x274b, 0x1dc7, 0x1932, 0x233a
	data	0x264e, 0x1b42, 0x1ab1, 0x25c0, 0x23ce, 0x19bc, 0x1d32, 0x26c6
	data	0x214c, 0x18b7, 0x2035, 0x274a, 0x1e48, 0x1933, 0x233a, 0x264d
	data	0x1bc2, 0x1ab1, 0x2540, 0x244e, 0x19bc, 0x1d32, 0x26c5, 0x214c
	data	0x1937, 0x1fb5, 0x26ca, 0x1ec8, 0x1934, 0x22ba, 0x264d, 0x1c43
	data	0x1ab2, 0x24bf, 0x244d, 0x1a3d, 0x1d32, 0x2645, 0x21cb, 0x1938
	data	0x1fb5, 0x26c9, 0x1ec8, 0x19b4, 0x2239, 0x25cc, 0x1c43, 0x1ab2
	data	0x24bf, 0x244d, 0x1abe, 0x1cb3, 0x2644, 0x21cb, 0x19b9, 0x1f35
	data	0x26c8, 0x1f48, 0x19b5, 0x2239, 0x25cb, 0x1cc3, 0x1ab3, 0x243e
	data	0x244c, 0x1abe, 0x1cb3, 0x25c3, 0x224b, 0x19b9, 0x1f35, 0x2648
	data	0x1f48, 0x19b5, 0x21b9, 0x25cb, 0x1d44, 0x1ab3, 0x243e, 0x244c
	data	0x1b3f, 0x1cb3, 0x25c3, 0x224b, 0x1a3a, 0x1f35, 0x2647, 0x1fc8
	data	0x19b6, 0x21b9, 0x25ca, 0x1d44, 0x1ab4, 0x23bd, 0x244c, 0x1b3f
	data	0x1cb4, 0x2542, 0x224b, 0x1a3a, 0x1eb5, 0x25c7, 0x1fc8, 0x1a37
	data	0x2139, 0x25ca, 0x1dc4, 0x1ab4, 0x233d, 0x244b, 0x1bc0, 0x1cb4
	data	0x24c2, 0x224b, 0x1abb, 0x1eb5, 0x25c6, 0x2048, 0x1a37, 0x20b8
	data	0x2549, 0x1dc5, 0x1ab5, 0x233d, 0x244b, 0x1c40, 0x1c34, 0x24c1
	data	0x22ca, 0x1abc, 0x1e36, 0x2545, 0x2048, 0x1a38, 0x20b8, 0x2549
	data	0x1e45, 0x1b35, 0x22bc, 0x244a, 0x1c40, 0x1c35, 0x2441, 0x22ca
	data	0x1b3c, 0x1e36, 0x2545, 0x20c8, 0x1ab8, 0x20b8, 0x2548, 0x1e45
	data	0x1b36, 0x22bc, 0x244a, 0x1cc1, 0x1c35, 0x2440, 0x22ca, 0x1b3d
	data	0x1e36, 0x2544, 0x20c8, 0x1ab9, 0x2038, 0x2548, 0x1ec5, 0x1b36
	data	0x223c, 0x2449, 0x1cc1, 0x1c35, 0x23c0, 0x22c9, 0x1bbd, 0x1e36
	data	0x24c4, 0x20c8, 0x1b39, 0x2038, 0x24c7, 0x1ec5, 0x1b37, 0x223c
	data	0x2449, 0x1d42, 0x1c36, 0x23bf, 0x22c9, 0x1bbe, 0x1db6, 0x24c3
	data	0x2148, 0x1b3a, 0x1fb8, 0x24c6, 0x1f45, 0x1b37, 0x21bb, 0x2448
	data	0x1d42, 0x1c36, 0x233f, 0x22c9, 0x1c3e, 0x1db7, 0x2443, 0x2148
	data	0x1b3a, 0x1fb8, 0x24c6, 0x1f45, 0x1bb8, 0x21bb, 0x2448, 0x1dc2
	data	0x1c37, 0x233f, 0x22c9, 0x1c3e, 0x1db7, 0x2442, 0x2148, 0x1bbb
	data	0x1fb8, 0x2445, 0x1fc5, 0x1bb8, 0x213b, 0x2447, 0x1dc2, 0x1c37
	data	0x22be, 0x22c8, 0x1cbf, 0x1db7, 0x23c2, 0x2147, 0x1bbb, 0x1f38
	data	0x2445, 0x1fc5, 0x1bb9, 0x213b, 0x23c7, 0x1e42, 0x1cb7, 0x22be
	data	0x22c8, 0x1cbf, 0x1db7, 0x23c2, 0x21c7, 0x1c3c, 0x1f39, 0x2445
	data	0x1fc5, 0x1bb9, 0x20bb, 0x23c7, 0x1e43, 0x1cb8, 0x223e, 0x22c7
	data	0x1d3f, 0x1db8, 0x2341, 0x21c7, 0x1c3c, 0x1f39, 0x23c4, 0x2045
	data	0x1c3a, 0x20bb, 0x23c6, 0x1ec3, 0x1cb8, 0x223e, 0x22c7, 0x1d40
	data	0x1db8, 0x2341, 0x21c7, 0x1c3d, 0x1f39, 0x23c4, 0x2045, 0x1c3a
	data	0x20bb, 0x23c6, 0x1ec3, 0x1cb9, 0x21be, 0x22c7, 0x1d40, 0x1db8
	data	0x22c0, 0x21c7, 0x1cbd, 0x1eb9, 0x23c3, 0x2045, 0x1c3b, 0x203b
	data	0x2345, 0x1ec3, 0x1cb9, 0x21bd, 0x22c6, 0x1dc0, 0x1db9, 0x22c0
	data	0x21c6, 0x1cbd, 0x1eb9, 0x2343, 0x2045, 0x1cbb, 0x203b, 0x2345
	data	0x1f43, 0x1cb9, 0x21bd, 0x22c6, 0x1dc0, 0x1db9, 0x22c0, 0x21c6
	data	0x1d3e, 0x1eb9, 0x2342, 0x20c5, 0x1cbb, 0x203b, 0x2344, 0x1f43
	data	0x1cba, 0x213d, 0x22c6, 0x1e41, 0x1db9, 0x2240, 0x21c6, 0x1d3e
	data	0x1eba, 0x22c2, 0x20c5, 0x1cbc, 0x203b, 0x2344, 0x1f43, 0x1d3a
	data	0x213d, 0x22c5, 0x1e41, 0x1dba, 0x223f, 0x21c5, 0x1d3e, 0x1eba
	data	0x22c2, 0x20c5, 0x1d3c, 0x1fbb, 0x22c4, 0x1fc3, 0x1d3b, 0x213d
	data	0x22c5, 0x1e41, 0x1dba, 0x21bf, 0x21c5, 0x1dbf, 0x1eba, 0x22c1
	data	0x20c5, 0x1d3d, 0x1fbb, 0x22c3, 0x1fc3, 0x1d3b, 0x20bd, 0x2245
	data	0x1ec1, 0x1dba, 0x21bf, 0x21c5, 0x1dbf, 0x1eba, 0x2241, 0x20c4
	data	0x1d3d, 0x1fbb, 0x22c3, 0x1fc3, 0x1d3b, 0x20bd, 0x2244, 0x1ec1
	data	0x1dbb, 0x21bf, 0x21c5, 0x1e3f, 0x1ebb, 0x2241, 0x20c4, 0x1dbd
	data	0x1fbb, 0x2243, 0x1fc3, 0x1dbc, 0x20bd, 0x2244, 0x1ec1, 0x1dbb
	data	0x213f, 0x21c4, 0x1e3f, 0x1ebb, 0x2241, 0x20c4, 0x1dbd, 0x1fbb
	data	0x2242, 0x1fc3, 0x1dbc, 0x203d, 0x2243, 0x1f41, 0x1e3b, 0x213f
	data	0x21c4, 0x1e40, 0x1ebb, 0x21c0, 0x20c4, 0x1dbe, 0x1f3c, 0x2242
	data	0x2043, 0x1dbc, 0x203d, 0x2243, 0x1f41, 0x1e3b, 0x213e, 0x21c4
	data	0x1e40, 0x1ebb, 0x21c0, 0x20c4, 0x1e3e, 0x1f3c, 0x2242, 0x2043
	data	0x1dbd, 0x203d, 0x21c3, 0x1f41, 0x1e3c, 0x213e, 0x21c3, 0x1ec0
	data	0x1ebc, 0x21c0, 0x20c3, 0x1e3e, 0x1f3c, 0x21c1, 0x2043, 0x1e3d
	data	0x203d, 0x21c3, 0x1f41, 0x1e3c, 0x20be, 0x21c3, 0x1ec0, 0x1ebc
	data	0x2140, 0x20c3, 0x1e3e, 0x1f3c, 0x21c1, 0x2042, 0x1e3d, 0x203d
	data	0x21c2, 0x1fc1, 0x1e3c, 0x20be, 0x2143, 0x1ec0, 0x1ebc, 0x2140
	data	0x20c3, 0x1e3f, 0x1f3c, 0x21c1, 0x2042, 0x1e3d, 0x203d, 0x21c2
	data	0x1fc1, 0x1e3d, 0x20be, 0x2143, 0x1f40, 0x1ebc, 0x2140, 0x20c3
	data	0x1ebf, 0x1f3c, 0x2141, 0x2042, 0x1e3e, 0x1fbd, 0x21c2, 0x1fc1
	data	0x1ebd, 0x20be, 0x2142, 0x1f40, 0x1ebd, 0x213f, 0x20c2, 0x1ebf
	data	0x1f3d, 0x2141, 0x2042, 0x1ebe, 0x1fbd, 0x2142, 0x1fc1, 0x1ebd
	data	0x203e, 0x2142, 0x1f40, 0x1ebd, 0x20bf, 0x20c2, 0x1ebf, 0x1f3d
	data	0x2140, 0x2042, 0x1ebe, 0x1fbd, 0x2141, 0x1fc1, 0x1ebd, 0x203e
	data	0x2142, 0x1f40, 0x1ebd, 0x20bf, 0x20c2, 0x1ebf, 0x1f3d, 0x2140
	data	0x2042, 0x1ebe, 0x1fbd, 0x2141, 0x1fc1, 0x1ebe, 0x203e, 0x2142
	data	0x1f40, 0x1f3d, 0x20bf, 0x20c2, 0x1f3f, 0x1f3d, 0x2140, 0x2042
	data	0x1ebe, 0x1fbe, 0x2141, 0x1fc1, 0x1ebe, 0x203e, 0x2141, 0x1fc0
	data	0x1f3d, 0x20bf, 0x20c2, 0x1f3f, 0x1f3d, 0x20c0, 0x2041, 0x1f3f
	data	0x1fbe, 0x2141, 0x2041, 0x1f3e, 0x203e, 0x20c1, 0x1fc0, 0x1f3e
	data	0x20bf, 0x20c1, 0x1f40, 0x1f3e, 0x20c0, 0x2041, 0x1f3f, 0x1fbe
	data	0x20c1, 0x2041, 0x1f3e, 0x203e, 0x20c1, 0x1fc0, 0x1f3e, 0x203f
	data	0x20c1, 0x1f40, 0x1f3e, 0x20c0, 0x2041, 0x1f3f, 0x1fbe, 0x20c0
	data	0x2041, 0x1f3e, 0x203e, 0x20c1, 0x1fc0, 0x1f3e, 0x203f, 0x20c1
	data	0x1f40, 0x1f3e, 0x20c0, 0x2041, 0x1f3f, 0x1fbe, 0x20c0, 0x2041
	data	0x1f3f, 0x203e, 0x20c1, 0x1fc0, 0x1f3e, 0x203f, 0x20c1, 0x1fc0
	data	0x1fbe, 0x20c0, 0x2041, 0x1f3f, 0x1fbe, 0x20c0, 0x2041, 0x1f3f
	data	0x203f, 0x20c1, 0x1fc0, 0x1f3e, 0x203f, 0x20c1, 0x1fc0, 0x1fbe
	data	0x2040, 0x2041, 0x1f3f, 0x1fbe, 0x20c0, 0x2041, 0x1f3f, 0x203f
	data	0x20c0, 0x1fc0, 0x1f3f, 0x203f, 0x2041, 0x1fc0, 0x1fbe, 0x203f
	data	0x2041, 0x1fbf, 0x1fbe, 0x2040, 0x2040, 0x1f3f, 0x1fbf, 0x20c0
	data	0x1fc0, 0x1fbf, 0x203f, 0x2040, 0x1fc0, 0x1fbf, 0x203f, 0x2040
	data	0x1fbf, 0x1fbf, 0x2040, 0x2040, 0x1fbf, 0x1fbf, 0x2040, 0x1fc0
	data	0x1fbf, 0x203f, 0x2040, 0x1fc0, 0x1fbf, 0x203f, 0x2040, 0x1fbf
	data	0x1fbf, 0x2040, 0x2040, 0x1fbf, 0x1fbf, 0x2040, 0x1fc0, 0x1fbf
	data	0x203f, 0x2040, 0x1fc0, 0x1fbf, 0x203f, 0x2040, 0x1fbf, 0x1fbf
	data	0x2040, 0x2040, 0x1fbf, 0x1fbf, 0x2040, 0x1fc0, 0x1fbf, 0x203f
	data	0x2040, 0x1fc0, 0x1fbf, 0x203f, 0x2040, 0x1fbf, 0x1fbf, 0x2040
	data	0x2040, 0x1fbf, 0x1fbf, 0x2040, 0x1fc0, 0x1fbf, 0x203f, 0x2040
	data	0x1fc0, 0x1fbf, 0x203f, 0x2040, 0x1fbf, 0x1fbf, 0x2040, 0x2040
	data	0x1f3f, 0x1e3d, 0x1d3b, 0x1c39, 0x1b37, 0x1a35, 0x1933, 0x1831
	data	0x172f, 0x162d, 0x152b, 0x1429, 0x1327, 0x1225, 0x1123, 0x1021
	data	0x0f1f, 0x0e1d, 0x0d1b, 0x0c19, 0x0b17, 0x0a15, 0x0913, 0x0811
	data	0x070f, 0x060d, 0x050b, 0x0409, 0x0307, 0x0205, 0x0103, 0x0001



; ------------------------------------------------------------------------
; Code Block
;
        ; Reset Vector
PAGE_0  org		0x0
	goto		MAIN_VECTOR


; ------------------------------------------------------------------------
; Interrupt Code Block
        org		0x4

	; Save interrupt state
	movwf		W_Temp
	swapf		STATUS,W
	clrf		STATUS
	movwf		STATUS_Temp
	movf		PCLATH,W
	movwf		PCLATH_Temp
	clrf		PCLATH
	movf		FSR,W
	movwf		FSR_Temp

	; Check for an incoming parallel character
	btfss		INTCON,INTF
	goto		ISR_CHECK_SIF_RX

	; Clear the interrupt
	bcf		INTCON,INTF

	; Save the ASCII value of the character into our buffer
	movlw		HIGH ASCII_TABLE	; Setup table high address
	movwf		FlashAddrHi
	movf		PORTA,W			; Get the data from the parallel port
	andlw		0x7F			;  convert to low address
	addlw		LOW ASCII_TABLE		;  along with table's low starting byte
	call		IsrReadProgram
	movlw		TxFifoStart		; Setup push pointer into TX FIFO
	addwf		TxPushPtr,W
	movwf		FSR
	movf		FlashDataLo,W		; Push translated character into TX FIFO
	movwf		INDF
	incf		TxPushPtr,W		; ++PushTxPtr
	andlw		TX_FIFO_MASK
	movwf		TxPushPtr

ISR_CHECK_SIF_RX
	; Check if we received a character from the EUART
	btfss		PIR1,RCIF
	goto		ISR_CHECK_SOUND

	; Check the RX STATUS
	btfsc		RCSTA,FERR		; Framing Error?
	goto		ISR_RX_FERR
	btfsc		RCSTA,OERR		; Overrun?
	goto		ISR_RX_OERR
	; Push the received character into our RX FIFO
	movlw		RxFifoStart
	addwf		RxPushPtr,W
	movwf		FSR
	movf		RCREG,W
	movwf		INDF
	incf		RxPushPtr,W
	andlw		RX_FIFO_MASK
	movwf		RxPushPtr
	goto		ISR_CHECK_SOUND

ISR_RX_OERR
	movf		RCREG,W			; Drain Entry 1 of EUART RX FIFO
ISR_RX_FERR
	movf		RCREG,W			; Drain Entry 2 of EUART RX FIFO (OERR)
						;   or badly framed character (FERR)
	bcf		RCSTA,CREN		; Reset the RX port and clear errors
	bsf		RCSTA,CREN

ISR_CHECK_SOUND
	; Check if we got a TIMER0 interrupt (potential sound sample)
	btfss		INTCON,T0IF
	goto		ISR_RESTORE
	bcf		INTCON,T0IF		; Clear timer interrupt

	; Check if we are outputting sound data
	btfss		Flags,BellInProgress
	goto		ISR_RESTORE
	btfsc		Flags,UseOddSample
	goto		ISR_SEND_ODD_SAMPLE
	; Setup even sample: start by getting both 7-bit values from program memory
	movlw		HIGH SOUND_TABLE
	addwf		SoundIndexHi,W
	movwf		FlashAddrHi
	movf		SoundIndexLo,W
	call		IsrReadProgram
	movf		FlashDataHi,W		; Get odd sample bits 5:0 from the
	movwf		SoundOddSample		;   upper program memory half
	rlf		FlashDataLo,W		;   Set C from lower half[7]
	rlf		SoundOddSample,F	; Rotate C into LSbit
	; Load the even sample into the PWM
	movlw		CCP1CON_MASK		; Preserve all bits but DC1B[1:0]
	andwf		CCP1CON,F		;   in CCP1CON
	swapf		FlashDataLo,W		; Even Sample[1:0]
	andlw		~CCP1CON_MASK		;   into CCP1CON[5:4]
	iorwf		CCP1CON,F
	rrf		FlashDataLo,F		; Load Even Sample[6:2]
	rrf		FlashDataLo,W		;   into CCPR1L[4:0]
	andlw		0x1F
	movwf		CCPR1L
	; Increment SoundIndex
	incfsz		SoundIndexLo,F		; ++Low Half = 0?
	goto		ISR_SETUP_ODD_SAMPLE	;   N: Don't need to increment high half
	incf		SoundIndexHi,F		;   Y: Inc high half
ISR_SETUP_ODD_SAMPLE
	; Setup for odd sample next time
	bsf		Flags,UseOddSample
	goto		ISR_RESTORE

ISR_SEND_ODD_SAMPLE
	; Send the odd sample we read in the previous sample period into the PWM
	movlw		CCP1CON_MASK		; Preserve all bits but DC1B[1:0]
	andwf		CCP1CON,F		;   in CCP1CON
	swapf		SoundOddSample,W	; SoundOddSample[1:0]
	andlw		~CCP1CON_MASK		;   into CCP1CON[5:4]
	iorwf		CCP1CON,F
	rrf		SoundOddSample,F	; Load SoundOddSample[6:2]
	rrf		SoundOddSample,W	;   into CCPR1L[4:0]
	andlw		0x1F
	movwf		CCPR1L

	; Check if we are done with the sound
	movlw		HIGH SOUND_TABLE_LEN
	subwf		SoundIndexHi,W
	btfss		STATUS,Z		; Hi Half matches end value?
	goto		ISR_SETUP_EVEN_SAMPLE	;   N: Setup for next even sample
	movlw		LOW SOUND_TABLE_LEN	;   Y: Check low half
	subwf		SoundIndexLo,W
	btfss		STATUS,Z		; Low Half matches end value?
	goto		ISR_SETUP_EVEN_SAMPLE	;   N: Setup for next even sample
	bcf		Flags,BellInProgress	;   Y: End Sound generation
	goto		ISR_RESTORE

ISR_SETUP_EVEN_SAMPLE
	bcf		Flags,UseOddSample

ISR_RESTORE
	; Restore interrupt state
	movf		FSR_Temp,W
	movwf		FSR
	movf		PCLATH_Temp,W
	movwf		PCLATH
	swapf		STATUS_Temp,W
	movwf		STATUS
	swapf		W_Temp,F
	swapf		W_Temp,W
	; Return from the interrupt (and re-enable interrupts)
	retfie


; ------------------------------------------------------------------------
; Main Code Block
;
MAIN_VECTOR
	call		InitPic
	call		InitVars

	; Enable interrupts after all other initialization
	movlw		INTCON_SETUP
	movwf		INTCON

MAIN_LOOP
	clrwdt

	; Echo shift input to output
	call		EvalShift

	; Process any received characters
	movf		RxPushPtr,W
	subwf		RxPopPtr,W
	btfss		STATUS,Z		; RxPushPtr = RxPopPtr?
	call		ProcessRxByte		;   N: Pop and process one byte
						;   Y: Continue

	; Only look for data in our TX FIFO or the repeat buffer to send if
	; the EUART is ready
	btfss		PIR1,TXIF		; TXIF = 1? (TXREG available)
	goto		MAIN_LOOP		;   N: nothing to do now

	; Check if there is data available to process
	movf		TxPushPtr,W
	subwf		TxPopPtr,W
	btfss		STATUS,Z		; TxPushPtr = TxPopPtr?
	call		ProcessTxByte		;   N: Pop and process one byte
						;   Y: Continue

	; Check if we are repeating a character and it is time to repeat it
	btfss		Flags,RepeatActive	; Repeating?
	goto		MAIN_LOOP		;   N: Done for now
	btfss		PIR1,TMR1IF		;   Y: Timer expired?
	goto		MAIN_LOOP		;      N: Done for now
	bcf		PIR1,TMR1IF		;      Y: Repeat (clear timer for next time)
	movf		RepeatChar,W
	movwf		TXREG
	goto		MAIN_LOOP


; ------------------------------------------------------------------------
; Subroutines
;

; InitPic: Initialize processor control registers
;
InitPic
	; Bank 0
	movlw		STATUS_SETUP
	movwf		STATUS
	clrf		FSR
	movlw		PORTA_INIT
	movwf		PORTA
	movlw		PORTB_INIT
	movwf		PORTB
	movlw		PORTC_INIT
	movwf		PORTC
	clrf		TMR1L
	clrf		TMR1H
	movlw		T1CON_INIT
	movwf		T1CON
	movlw		T2CON_INIT
	movwf		T2CON
	clrf		CCPR1L
	movlw		CCP1CON_INIT
	movwf		CCP1CON
	movlw		RCSTA_INIT
	movwf		RCSTA

	; Bank 1
	bsf		STATUS,RP0
	movlw		OPTION_SETUP
	movwf		OPTION_REG
	movlw		PORTA_OE
	movwf		TRISA
	movlw		PORTB_OE
	movwf		TRISB
	movlw		PORTC_OE
	movwf		TRISC
	movlw		PIE1_INIT
	movwf		PIE1
	movlw		OSCCON_SETUP
	movwf		OSCCON
	movlw		PR2_INIT
	movwf		PR2
	movlw		WPUB_INIT
	movwf		WPUB
	movlw		IOCB_INIT
	movwf		IOCB
	movlw		TXSTA_INIT
	movwf		TXSTA
	movlw		(SPBRG_INIT & 0xFF)
	movwf		SPBRG
	movlw		(SPBRG_INIT >> 8)
	movwf		SPBRGH
	bcf		STATUS,RP0

	; Bank 2

	; Bank 3
	bsf		STATUS,RP0
	bsf		STATUS,RP1
	movlw		ANSEL_INIT
	movwf		ANSEL
	movlw		ANSELH_INIT
	movwf		ANSELH
	movlw		BAUDCTL_INIT
	movwf		BAUDCTL
	bcf		STATUS,RP0
	bcf		STATUS,RP1

	return


; InitVars: Initialize program variables
;
InitVars
	clrf		TxPushPtr
	clrf		TxPopPtr
	clrf		RxPushPtr
	clrf		RxPopPtr
	clrf		Flags
	return


; IsrReadProgram: Read the program flash memory.  Based on the routine
; provided in section 10.1.4 of the specification.
;
; on entry: W contains the low-byte address to read (0 - 0xFF)
;           FlashAddrHi contains the high-byte address to read (0 - 0x0F)
; 
; on exit: FlashData contains the low 8-bits of data read
;          FlashDataHi contains the high 6-bits of data read
; 
; calling limit: designed to be called by the ISR
;
; Requires 17 cycles
; 
; Notes: none
IsrReadProgram 
	bsf		STATUS,RP1	; Bank 2
	movwf		EEADR		; Set the EEADR reg in bank 2
	movf		FlashAddrHi,W
	movwf		EEADRH		; Set the EEADRH reg
	bsf		STATUS,RP0	; Move to bank 3
	bsf		EECON1,EEPGD	; Point to program memory
	bsf		EECON1,RD
	nop				; Two required NOPs
	nop
	bcf		STATUS,RP0	; Back to bank 2
	movf		EEDAT,W		; Then copy data
	movwf		FlashDataLo
	movf		EEDATH,W
	movwf		FlashDataHi
	bcf		STATUS,RP1	; Back to bank 0
	return


; EvalShift: Evaluate the shift input and echo it to the output
;
EvalShift
	; Sample input
	bcf		Flags,ShiftDown	; Will be set if necessary
	btfsc		PORTB,SHIFT_IN
	bsf		Flags,ShiftDown

	; Echo to output
	btfsc		Flags,ShiftDown
	bsf		PORTC,SHIFT_OUT
	btfss		Flags,ShiftDown
	bcf		PORTC,SHIFT_OUT
	return


; ProcessRxByte: Process a byte from the RX FIFO looking to see if we see the BELL character.
; If so then setup a bell sound (unless one is already running)
;
ProcessRxByte
	; Pop a character and check if it is BELL_CHAR
	movlw		RxFifoStart		; Setup pop pointer
	addwf		RxPopPtr,W
	movwf		FSR
	movf		INDF,W
	sublw		BELL_CHAR
	btfss		STATUS,Z		; Character = BELL_CHAR?
	goto		PROCESS_RX_BYTE_RET	;   N: Skip this character
						;   Y: See if we should start a bell sound
	btfsc		Flags,BellInProgress	; Already generated a bell sound?
	goto		PROCESS_RX_BYTE_RET	;   Y: Skip this BELL_CHAR
						;   N: Setup for a bell sound
	clrf		SoundIndexHi
	clrf		SoundIndexLo
	bcf		Flags,UseOddSample
	bsf		Flags,BellInProgress

PROCESS_RX_BYTE_RET
	; Bump the pointer
	incf		RxPopPtr,W		; ++RxPopPtr
	andlw		RX_FIFO_MASK
	movwf		RxPopPtr
	return


; ProcessTxByte: Process a byte from the TX FIFO looking to see if it starts or stops
; a repeat process or is just a data byte to be sent
;
ProcessTxByte
	movlw		TxFifoStart		; Setup pop pointer
	addwf		TxPopPtr,W
	movwf		FSR
	movf		INDF,W			; Get the character
	sublw		REPEAT_CHAR
	btfss		STATUS,Z		; REPEAT_CHAR?
	goto		PROCESS_TX_SEND		;   N: Just send it out
						;   Y: Setup or end repeating
	btfsc		Flags,RepeatActive	; Are we currently repeating?
	goto		PROCESS_TX_END_REPEAT	;   Y: End repeating
						;   N: Setup to repeat
	clrf		TMR1L			; Clear the repeat timer
	clrf		TMR1H
	bcf		PIR1,TMR1IF
	bsf		Flags,RepeatActive	; Note we are repeating
	goto		PROCESS_TX_BYTE_RET

PROCESS_TX_END_REPEAT
	; Stop repeating
	bcf		Flags,RepeatActive
	goto		PROCESS_TX_BYTE_RET

PROCESS_TX_SEND
	; Just send the byte (and cancel any current repeat functions)
	movf		INDF,W
	movwf		TXREG
	movwf		RepeatChar		; In case the next character is REPEAT_CHAR
	bcf		Flags,RepeatActive

PROCESS_TX_BYTE_RET
	; Bump the pointer
	incf		TxPopPtr,W		; ++TxPopPtr
	andlw		TX_FIFO_MASK
	movwf		TxPopPtr
	return

	end

